==> matrice-1.0.98604-py3-none-any.whl: matrice/deploy/utils/post_processing/processor.py <==
"""
Main post-processing processor with unified, clean API.

This module provides the main PostProcessor class that serves as the entry point
for all post-processing operations. It manages use cases, configurations, and
provides both simple and advanced processing interfaces.
"""

from typing import Any, Dict, List, Optional, Union
from pathlib import Path
import logging
import time

from .core.base import (
    ProcessingResult, 
    ProcessingContext, 
    ProcessingStatus,
    registry
)
from .core.config import (
    BaseConfig, 
    PeopleCountingConfig, 
    CustomerServiceConfig,
    config_manager,
    ConfigValidationError
)
from .usecases import (
    PeopleCountingUseCase,
    CustomerServiceUseCase,
    AdvancedCustomerServiceUseCase,
    LicensePlateUseCase,
    VideoColorClassificationUseCase,
    PPEComplianceUseCase
)

logger = logging.getLogger(__name__)


class PostProcessor:
    """
    Unified post-processing interface with clean API and comprehensive functionality.
    
    This processor provides a simple yet powerful interface for processing model outputs
    with various use cases, centralized configuration management, and comprehensive
    error handling.
    
    Examples:
        # Simple usage
        processor = PostProcessor()
        result = processor.process_simple(
            raw_results, "people_counting", 
            confidence_threshold=0.6,
            zones={"entrance": [[0, 0], [100, 0], [100, 100], [0, 100]]}
        )
        
        # Configuration-based usage
        config = processor.create_config("people_counting", confidence_threshold=0.5)
        result = processor.process(raw_results, config)
        
        # File-based configuration
        result = processor.process_from_file(raw_results, "config.json")
    """
    
    def __init__(self):
        """Initialize the PostProcessor with registered use cases."""
        self._statistics = {
            "total_processed": 0,
            "successful": 0,
            "failed": 0,
            "total_processing_time": 0.0
        }
        
        # Register available use cases
        self._register_use_cases()
    
    def _register_use_cases(self) -> None:
        """Register all available use cases."""
        # Register people counting use case
        registry.register_use_case("general", "people_counting", PeopleCountingUseCase)
        
        # Register customer service use case
        registry.register_use_case("sales", "customer_service", CustomerServiceUseCase)
        
        # Register advanced customer service use case
        registry.register_use_case("sales", "advanced_customer_service", AdvancedCustomerServiceUseCase)
        
        # Register license plate detection use case
        registry.register_use_case("vehicle", "license_plate_detection", LicensePlateUseCase)
        
        registry.register_use_case("general", "video_color_classification", VideoColorClassificationUseCase)
        
        # Register PPE compliance use case
        registry.register_use_case("ppe", "ppe_compliance_detection", PPEComplianceUseCase)

        registry.register_use_case("vehicle", "vehicle_monitoring", VehicleMonitoringUseCase)
        
        logger.debug("Registered use cases with registry")
    
    def process(self, data: Any, config: Union[BaseConfig, Dict[str, Any], str, Path], input_bytes: Optional[bytes] = None,
                context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """
        Process data using the specified configuration.
        
        Args:
            data: Raw model output (detection, tracking, classification results)
            config: Configuration object, dict, or path to config file
            context: Optional processing context
            
        Returns:
            ProcessingResult: Standardized result object
        """
        start_time = time.time()
        
        try:
            # Parse configuration
            parsed_config = self._parse_config(config)
            
            # Get appropriate use case
            use_case_class = registry.get_use_case(parsed_config.category, parsed_config.usecase)
            if not use_case_class:
                return self._create_error_result(
                    f"Use case '{parsed_config.category}/{parsed_config.usecase}' not found",
                    "UseCaseNotFound",
                    parsed_config.usecase,
                    parsed_config.category,
                    context
                )
            
            # Instantiate use case
            use_case = use_case_class()
            
            # Create context if not provided
            if context is None:
                context = ProcessingContext()
            
            # Process with use case
            if use_case_class == VideoColorClassificationUseCase:
                result = use_case.process(data, parsed_config, input_bytes, context)
            else:
                result = use_case.process(data, parsed_config, context)
            
            # Add processing time
            result.processing_time = time.time() - start_time
            
            # Update statistics
            self._update_statistics(result)
            
            return result
            
        except Exception as e:
            processing_time = time.time() - start_time
            logger.error(f"Processing failed: {str(e)}", exc_info=True)
            
            error_result = self._create_error_result(
                str(e),
                type(e).__name__,
                context=context
            )
            error_result.processing_time = processing_time
            
            # Update statistics
            self._update_statistics(error_result)
            
            return error_result
    
    def process_simple(self, data: Any, usecase: str, 
                      category: Optional[str] = None,
                      context: Optional[ProcessingContext] = None,
                      **config_params) -> ProcessingResult:
        """
        Simple processing interface for quick use cases.
        
        Args:
            data: Raw model output
            usecase: Use case name ('people_counting', 'customer_service', etc.)
            category: Use case category (auto-detected if not provided)
            context: Optional processing context
            **config_params: Configuration parameters
            
        Returns:
            ProcessingResult: Standardized result object
        """
        try:
            # Auto-detect category if not provided
            if category is None:
                if usecase == "people_counting":
                    category = "general"
                elif usecase == "customer_service":
                    category = "sales"
                else:
                    category = "general"  # Default fallback
            
            # Create configuration
            config = self.create_config(usecase, category=category, **config_params)
            return self.process(data, config, context)
            
        except Exception as e:
            logger.error(f"Simple processing failed: {str(e)}", exc_info=True)
            return self._create_error_result(
                str(e),
                type(e).__name__,
                usecase,
                category or "general",
                context
            )
    
    def process_from_file(self, data: Any, config_file: Union[str, Path],
                         context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """
        Process data using configuration from file.
        
        Args:
            data: Raw model output
            config_file: Path to configuration file (JSON or YAML)
            context: Optional processing context
            
        Returns:
            ProcessingResult: Standardized result object
        """
        try:
            config = config_manager.load_from_file(config_file)
            return self.process(data, config, context)
            
        except Exception as e:
            logger.error(f"File-based processing failed: {str(e)}", exc_info=True)
            return self._create_error_result(
                f"Failed to process with config file: {str(e)}",
                type(e).__name__,
                context=context
            )
    
    def create_config(self, usecase: str, category: str = "general", **kwargs) -> BaseConfig:
        """
        Create a validated configuration object.
        
        Args:
            usecase: Use case name
            category: Use case category
            **kwargs: Configuration parameters
            
        Returns:
            BaseConfig: Validated configuration object
        """
        return config_manager.create_config(usecase, category=category, **kwargs)
    
    def load_config(self, file_path: Union[str, Path]) -> BaseConfig:
        """Load configuration from file."""
        return config_manager.load_from_file(file_path)
    
    def save_config(self, config: BaseConfig, file_path: Union[str, Path], 
                   format: str = "json") -> None:
        """Save configuration to file."""
        config_manager.save_to_file(config, file_path, format)
    
    def get_config_template(self, usecase: str) -> Dict[str, Any]:
        """Get configuration template for a use case."""
        return config_manager.get_config_template(usecase)
    
    def list_available_usecases(self) -> Dict[str, List[str]]:
        """List all available use cases by category."""
        return registry.list_use_cases()
    
    def get_supported_usecases(self) -> List[str]:
        """Get list of supported use case names."""
        return config_manager.list_supported_usecases()
    
    def get_use_case_schema(self, usecase: str, category: str = "general") -> Dict[str, Any]:
        """
        Get JSON schema for a use case configuration.
        
        Args:
            usecase: Use case name
            category: Use case category
            
        Returns:
            Dict[str, Any]: JSON schema for the use case
        """
        use_case_class = registry.get_use_case(category, usecase)
        if not use_case_class:
            raise ValueError(f"Use case '{category}/{usecase}' not found")
        
        use_case = use_case_class()
        return use_case.get_config_schema()
    
    def validate_config(self, config: Union[BaseConfig, Dict[str, Any]]) -> List[str]:
        """
        Validate a configuration object or dictionary.
        
        Args:
            config: Configuration to validate
            
        Returns:
            List[str]: List of validation errors (empty if valid)
        """
        try:
            if isinstance(config, dict):
                usecase = config.get('usecase')
                if not usecase:
                    return ["Configuration must specify 'usecase'"]
                
                category = config.get('category', 'general')
                parsed_config = config_manager.create_config(usecase, category=category, **config)
                return parsed_config.validate()
            elif isinstance(config, BaseConfig):
                return config.validate()
            else:
                return [f"Invalid configuration type: {type(config)}"]
                
        except Exception as e:
            return [f"Configuration validation failed: {str(e)}"]
    
    def get_statistics(self) -> Dict[str, Any]:
        """
        Get processing statistics.
        
        Returns:
            Dict[str, Any]: Processing statistics
        """
        stats = self._statistics.copy()
        if stats["total_processed"] > 0:
            stats["success_rate"] = stats["successful"] / stats["total_processed"]
            stats["failure_rate"] = stats["failed"] / stats["total_processed"]
            stats["average_processing_time"] = stats["total_processing_time"] / stats["total_processed"]
        else:
            stats["success_rate"] = 0.0
            stats["failure_rate"] = 0.0
            stats["average_processing_time"] = 0.0
        
        return stats
    
    def reset_statistics(self) -> None:
        """Reset processing statistics."""
        self._statistics = {
            "total_processed": 0,
            "successful": 0,
            "failed": 0,
            "total_processing_time": 0.0
        }
    
    def _parse_config(self, config: Union[BaseConfig, Dict[str, Any], str, Path]) -> BaseConfig:
        """Parse configuration from various input formats."""
        if isinstance(config, BaseConfig):
            return config
        elif isinstance(config, dict):
            usecase = config.get('usecase')
            if not usecase:
                raise ValueError("Configuration dict must contain 'usecase' key")
            
            category = config.get('category', 'general')
            return config_manager.create_config(usecase, category=category, **config)
        elif isinstance(config, (str, Path)):
            return config_manager.load_from_file(config)
        else:
            raise ValueError(f"Unsupported config type: {type(config)}")
    
    def _create_error_result(self, message: str, error_type: str = "ProcessingError",
                            usecase: str = "", category: str = "",
                            context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """Create an error result."""
        result = ProcessingResult(
            data={},
            status=ProcessingStatus.ERROR,
            usecase=usecase,
            category=category,
            context=context,
            error_message=message,
            error_type=error_type,
            summary=f"Processing failed: {message}"
        )
        
        if context:
            result.processing_time = context.processing_time or 0.0
        
        return result
    
    def _update_statistics(self, result: ProcessingResult) -> None:
        """Update processing statistics."""
        self._statistics["total_processed"] += 1
        self._statistics["total_processing_time"] += result.processing_time
        
        if result.is_success():
            self._statistics["successful"] += 1
        else:
            self._statistics["failed"] += 1


# Convenience functions for backward compatibility and simple usage
def process_simple(data: Any, usecase: str, category: Optional[str] = None, **config) -> ProcessingResult:
    """
    Simple processing function for quick use cases.
    
    Args:
        data: Raw model output
        usecase: Use case name ('people_counting', 'customer_service', etc.)
        category: Use case category (auto-detected if not provided)
        **config: Configuration parameters
        
    Returns:
        ProcessingResult: Standardized result object
    """
    processor = PostProcessor()
    return processor.process_simple(data, usecase, category, **config)


def create_config_template(usecase: str) -> Dict[str, Any]:
    """
    Create a configuration template for a use case.
    
    Args:
        usecase: Use case name
        
    Returns:
        Dict[str, Any]: Configuration template
    """
    processor = PostProcessor()
    return processor.get_config_template(usecase)


def list_available_usecases() -> Dict[str, List[str]]:
    """
    List all available use cases.
    
    Returns:
        Dict[str, List[str]]: Available use cases by category
    """
    processor = PostProcessor()
    return processor.list_available_usecases()


def validate_config(config: Union[BaseConfig, Dict[str, Any]]) -> List[str]:
    """
    Validate a configuration.
    
    Args:
        config: Configuration to validate
        
    Returns:
        List[str]: List of validation errors
    """
    processor = PostProcessor()
    return processor.validate_config(config)

==> matrice-1.0.98606-py3-none-any.whl: matrice/deploy/utils/post_processing/processor.py <==
"""
Main post-processing processor with unified, clean API.

This module provides the main PostProcessor class that serves as the entry point
for all post-processing operations. It manages use cases, configurations, and
provides both simple and advanced processing interfaces.
"""

from typing import Any, Dict, List, Optional, Union
from pathlib import Path
import logging
import time

from .core.base import (
    ProcessingResult, 
    ProcessingContext, 
    ProcessingStatus,
    registry
)
from .core.config import (
    BaseConfig, 
    PeopleCountingConfig, 
    CustomerServiceConfig,
    config_manager,
    ConfigValidationError
)
from .usecases import (
    PeopleCountingUseCase,
    CustomerServiceUseCase,
    AdvancedCustomerServiceUseCase,
    LicensePlateUseCase,
    VideoColorClassificationUseCase,
    PPEComplianceUseCase,
    VehicleMonitoringUseCase
)

logger = logging.getLogger(__name__)


class PostProcessor:
    """
    Unified post-processing interface with clean API and comprehensive functionality.
    
    This processor provides a simple yet powerful interface for processing model outputs
    with various use cases, centralized configuration management, and comprehensive
    error handling.
    
    Examples:
        # Simple usage
        processor = PostProcessor()
        result = processor.process_simple(
            raw_results, "people_counting", 
            confidence_threshold=0.6,
            zones={"entrance": [[0, 0], [100, 0], [100, 100], [0, 100]]}
        )
        
        # Configuration-based usage
        config = processor.create_config("people_counting", confidence_threshold=0.5)
        result = processor.process(raw_results, config)
        
        # File-based configuration
        result = processor.process_from_file(raw_results, "config.json")
    """
    
    def __init__(self):
        """Initialize the PostProcessor with registered use cases."""
        self._statistics = {
            "total_processed": 0,
            "successful": 0,
            "failed": 0,
            "total_processing_time": 0.0
        }
        
        # Register available use cases
        self._register_use_cases()
    
    def _register_use_cases(self) -> None:
        """Register all available use cases."""
        # Register people counting use case
        registry.register_use_case("general", "people_counting", PeopleCountingUseCase)
        
        # Register customer service use case
        registry.register_use_case("sales", "customer_service", CustomerServiceUseCase)
        
        # Register advanced customer service use case
        registry.register_use_case("sales", "advanced_customer_service", AdvancedCustomerServiceUseCase)
        
        # Register license plate detection use case
        registry.register_use_case("vehicle", "license_plate_detection", LicensePlateUseCase)
        
        registry.register_use_case("general", "video_color_classification", VideoColorClassificationUseCase)
        
        # Register PPE compliance use case
        registry.register_use_case("ppe", "ppe_compliance_detection", PPEComplianceUseCase)

        registry.register_use_case("vehicle", "vehicle_monitoring", VehicleMonitoringUseCase)
        
        logger.debug("Registered use cases with registry")
    
    def process(self, data: Any, config: Union[BaseConfig, Dict[str, Any], str, Path], input_bytes: Optional[bytes] = None,
                context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """
        Process data using the specified configuration.
        
        Args:
            data: Raw model output (detection, tracking, classification results)
            config: Configuration object, dict, or path to config file
            context: Optional processing context
            
        Returns:
            ProcessingResult: Standardized result object
        """
        start_time = time.time()
        
        try:
            # Parse configuration
            parsed_config = self._parse_config(config)
            
            # Get appropriate use case
            use_case_class = registry.get_use_case(parsed_config.category, parsed_config.usecase)
            if not use_case_class:
                return self._create_error_result(
                    f"Use case '{parsed_config.category}/{parsed_config.usecase}' not found",
                    "UseCaseNotFound",
                    parsed_config.usecase,
                    parsed_config.category,
                    context
                )
            
            # Instantiate use case
            use_case = use_case_class()
            
            # Create context if not provided
            if context is None:
                context = ProcessingContext()
            
            # Process with use case
            if use_case_class == VideoColorClassificationUseCase:
                result = use_case.process(data, parsed_config, input_bytes=input_bytes, context=context)
            else:
                result = use_case.process(data, parsed_config, context)
            
            # Add processing time
            result.processing_time = time.time() - start_time
            
            # Update statistics
            self._update_statistics(result)
            
            return result
            
        except Exception as e:
            processing_time = time.time() - start_time
            logger.error(f"Processing failed: {str(e)}", exc_info=True)
            
            error_result = self._create_error_result(
                str(e),
                type(e).__name__,
                context=context
            )
            error_result.processing_time = processing_time
            
            # Update statistics
            self._update_statistics(error_result)
            
            return error_result
    
    def process_simple(self, data: Any, usecase: str, 
                      category: Optional[str] = None,
                      context: Optional[ProcessingContext] = None,
                      **config_params) -> ProcessingResult:
        """
        Simple processing interface for quick use cases.
        
        Args:
            data: Raw model output
            usecase: Use case name ('people_counting', 'customer_service', etc.)
            category: Use case category (auto-detected if not provided)
            context: Optional processing context
            **config_params: Configuration parameters
            
        Returns:
            ProcessingResult: Standardized result object
        """
        try:
            # Auto-detect category if not provided
            if category is None:
                if usecase == "people_counting":
                    category = "general"
                elif usecase == "customer_service":
                    category = "sales"
                else:
                    category = "general"  # Default fallback
            
            # Create configuration
            config = self.create_config(usecase, category=category, **config_params)
            return self.process(data, config, context)
            
        except Exception as e:
            logger.error(f"Simple processing failed: {str(e)}", exc_info=True)
            return self._create_error_result(
                str(e),
                type(e).__name__,
                usecase,
                category or "general",
                context
            )
    
    def process_from_file(self, data: Any, config_file: Union[str, Path],
                         context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """
        Process data using configuration from file.
        
        Args:
            data: Raw model output
            config_file: Path to configuration file (JSON or YAML)
            context: Optional processing context
            
        Returns:
            ProcessingResult: Standardized result object
        """
        try:
            config = config_manager.load_from_file(config_file)
            return self.process(data, config, context)
            
        except Exception as e:
            logger.error(f"File-based processing failed: {str(e)}", exc_info=True)
            return self._create_error_result(
                f"Failed to process with config file: {str(e)}",
                type(e).__name__,
                context=context
            )
    
    def create_config(self, usecase: str, category: str = "general", **kwargs) -> BaseConfig:
        """
        Create a validated configuration object.
        
        Args:
            usecase: Use case name
            category: Use case category
            **kwargs: Configuration parameters
            
        Returns:
            BaseConfig: Validated configuration object
        """
        return config_manager.create_config(usecase, category=category, **kwargs)
    
    def load_config(self, file_path: Union[str, Path]) -> BaseConfig:
        """Load configuration from file."""
        return config_manager.load_from_file(file_path)
    
    def save_config(self, config: BaseConfig, file_path: Union[str, Path], 
                   format: str = "json") -> None:
        """Save configuration to file."""
        config_manager.save_to_file(config, file_path, format)
    
    def get_config_template(self, usecase: str) -> Dict[str, Any]:
        """Get configuration template for a use case."""
        return config_manager.get_config_template(usecase)
    
    def list_available_usecases(self) -> Dict[str, List[str]]:
        """List all available use cases by category."""
        return registry.list_use_cases()
    
    def get_supported_usecases(self) -> List[str]:
        """Get list of supported use case names."""
        return config_manager.list_supported_usecases()
    
    def get_use_case_schema(self, usecase: str, category: str = "general") -> Dict[str, Any]:
        """
        Get JSON schema for a use case configuration.
        
        Args:
            usecase: Use case name
            category: Use case category
            
        Returns:
            Dict[str, Any]: JSON schema for the use case
        """
        use_case_class = registry.get_use_case(category, usecase)
        if not use_case_class:
            raise ValueError(f"Use case '{category}/{usecase}' not found")
        
        use_case = use_case_class()
        return use_case.get_config_schema()
    
    def validate_config(self, config: Union[BaseConfig, Dict[str, Any]]) -> List[str]:
        """
        Validate a configuration object or dictionary.
        
        Args:
            config: Configuration to validate
            
        Returns:
            List[str]: List of validation errors (empty if valid)
        """
        try:
            if isinstance(config, dict):
                usecase = config.get('usecase')
                if not usecase:
                    return ["Configuration must specify 'usecase'"]
                
                category = config.get('category', 'general')
                parsed_config = config_manager.create_config(usecase, category=category, **config)
                return parsed_config.validate()
            elif isinstance(config, BaseConfig):
                return config.validate()
            else:
                return [f"Invalid configuration type: {type(config)}"]
                
        except Exception as e:
            return [f"Configuration validation failed: {str(e)}"]
    
    def get_statistics(self) -> Dict[str, Any]:
        """
        Get processing statistics.
        
        Returns:
            Dict[str, Any]: Processing statistics
        """
        stats = self._statistics.copy()
        if stats["total_processed"] > 0:
            stats["success_rate"] = stats["successful"] / stats["total_processed"]
            stats["failure_rate"] = stats["failed"] / stats["total_processed"]
            stats["average_processing_time"] = stats["total_processing_time"] / stats["total_processed"]
        else:
            stats["success_rate"] = 0.0
            stats["failure_rate"] = 0.0
            stats["average_processing_time"] = 0.0
        
        return stats
    
    def reset_statistics(self) -> None:
        """Reset processing statistics."""
        self._statistics = {
            "total_processed": 0,
            "successful": 0,
            "failed": 0,
            "total_processing_time": 0.0
        }
    
    def _parse_config(self, config: Union[BaseConfig, Dict[str, Any], str, Path]) -> BaseConfig:
        """Parse configuration from various input formats."""
        if isinstance(config, BaseConfig):
            return config
        elif isinstance(config, dict):
            usecase = config.get('usecase')
            if not usecase:
                raise ValueError("Configuration dict must contain 'usecase' key")
            
            category = config.get('category', 'general')
            return config_manager.create_config(usecase, category=category, **config)
        elif isinstance(config, (str, Path)):
            return config_manager.load_from_file(config)
        else:
            raise ValueError(f"Unsupported config type: {type(config)}")
    
    def _create_error_result(self, message: str, error_type: str = "ProcessingError",
                            usecase: str = "", category: str = "",
                            context: Optional[ProcessingContext] = None) -> ProcessingResult:
        """Create an error result."""
        result = ProcessingResult(
            data={},
            status=ProcessingStatus.ERROR,
            usecase=usecase,
            category=category,
            context=context,
            error_message=message,
            error_type=error_type,
            summary=f"Processing failed: {message}"
        )
        
        if context:
            result.processing_time = context.processing_time or 0.0
        
        return result
    
    def _update_statistics(self, result: ProcessingResult) -> None:
        """Update processing statistics."""
        self._statistics["total_processed"] += 1
        self._statistics["total_processing_time"] += result.processing_time
        
        if result.is_success():
            self._statistics["successful"] += 1
        else:
            self._statistics["failed"] += 1


# Convenience functions for backward compatibility and simple usage
def process_simple(data: Any, usecase: str, category: Optional[str] = None, **config) -> ProcessingResult:
    """
    Simple processing function for quick use cases.
    
    Args:
        data: Raw model output
        usecase: Use case name ('people_counting', 'customer_service', etc.)
        category: Use case category (auto-detected if not provided)
        **config: Configuration parameters
        
    Returns:
        ProcessingResult: Standardized result object
    """
    processor = PostProcessor()
    return processor.process_simple(data, usecase, category, **config)


def create_config_template(usecase: str) -> Dict[str, Any]:
    """
    Create a configuration template for a use case.
    
    Args:
        usecase: Use case name
        
    Returns:
        Dict[str, Any]: Configuration template
    """
    processor = PostProcessor()
    return processor.get_config_template(usecase)


def list_available_usecases() -> Dict[str, List[str]]:
    """
    List all available use cases.
    
    Returns:
        Dict[str, List[str]]: Available use cases by category
    """
    processor = PostProcessor()
    return processor.list_available_usecases()


def validate_config(config: Union[BaseConfig, Dict[str, Any]]) -> List[str]:
    """
    Validate a configuration.
    
    Args:
        config: Configuration to validate
        
    Returns:
        List[str]: List of validation errors
    """
    processor = PostProcessor()
    return processor.validate_config(config)
